@@ -1,30 +1,17 @@
-/*  This file is part of the KDE libraries
+/*
+    This file is part of the KDE libraries
 
     This class was originally inspired by Torben Weis'
     fileentry.cpp for KFM II.
 
-    Copyright (C) 1997 Sven Radej <sven.radej@iname.com>
-    Copyright (c) 1999 Patrick Ward <PAT_WARD@HP-USA-om5.om.hp.com>
-    Copyright (c) 1999 Preston Brown <pbrown@kde.org>
+    SPDX-FileCopyrightText: 1997 Sven Radej <sven.radej@iname.com>
+    SPDX-FileCopyrightText: 1999 Patrick Ward <PAT_WARD@HP-USA-om5.om.hp.com>
+    SPDX-FileCopyrightText: 1999 Preston Brown <pbrown@kde.org>
 
     Completely re-designed:
-    Copyright (c) 2000,2001 Dawit Alemayehu <adawit@kde.org>
+    SPDX-FileCopyrightText: 2000, 2001 Dawit Alemayehu <adawit@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License (LGPL) as published by the Free Software Foundation;
-    either version 2 of the License, or (at your option) any later
-    version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KLINEEDIT_H
@@ -35,6 +22,7 @@
 #include <kcompletion_export.h>
 
 #include <QLineEdit>
+#include <memory>
 
 class QAction;
 class QMenu;
@@ -63,7 +51,7 @@ class KLineEditPrivate;
  * user in filling out the remaining text. The text rotation signal is
  * intended to be used to iterate through the list of all possible matches
  * whenever there is more than one match for the entered text. The
- * @p returnPressed( const QString& ) signals are the same as QLineEdit's
+ * @c returnPressed( const QString& ) signals are the same as QLineEdit's
  * except it provides the current text in the widget as its argument whenever
  * appropriate.
  *
@@ -85,10 +73,10 @@ class KLineEditPrivate;
  * individual key-bindings by using setKeyBinding() with the default
  * second argument.
  *
- * If @p EchoMode for this widget is set to something other than @p QLineEdit::Normal,
+ * If @c EchoMode for this widget is set to something other than @c QLineEdit::Normal,
  * the completion mode will always be defaulted to CompletionNone.
  * This is done purposefully to guard against protected entries such as passwords being
- * cached in KCompletion's list. Hence, if the @p EchoMode is not QLineEdit::Normal, the
+ * cached in KCompletion's list. Hence, if the @c EchoMode is not QLineEdit::Normal, the
  * completion mode is automatically disabled.
  *
  * A read-only KLineEdit will have the same background color as a
@@ -157,18 +145,20 @@ class KCOMPLETION_EXPORT KLineEdit : public QLineEdit, public KCompletionBase //
 
     Q_OBJECT
     Q_DECLARE_PRIVATE(KLineEdit)
-#ifndef KCOMPLETION_NO_DEPRECATED
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(4, 5)
     Q_PROPERTY(bool contextMenuEnabled READ isContextMenuEnabled WRITE setContextMenuEnabled)
 #endif
-#ifndef KCOMPLETION_NO_DEPRECATED
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 0)
     Q_PROPERTY(bool urlDropsEnabled READ urlDropsEnabled WRITE setUrlDropsEnabled)
 #endif
     Q_PROPERTY(bool trapEnterKeyEvent READ trapReturnKey WRITE setTrapReturnKey)
     Q_PROPERTY(bool squeezedTextEnabled READ isSqueezedTextEnabled WRITE setSqueezedTextEnabled)
-#ifndef KCOMPLETION_NO_DEPRECATED
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 0)
     Q_PROPERTY(QString clickMessage READ clickMessage WRITE setClickMessage)
 #endif
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 46)
     Q_PROPERTY(bool showClearButton READ isClearButtonShown WRITE setClearButtonShown)
+#endif
     Q_PROPERTY(bool passwordMode READ passwordMode WRITE setPasswordMode)
 
 public:
@@ -191,7 +181,7 @@ public:
     /**
      *  Destructor.
      */
-    virtual ~KLineEdit();
+    ~KLineEdit() override;
 
     /**
      * Sets @p url into the lineedit. It uses QUrl::toDisplayString() so
@@ -207,7 +197,7 @@ public:
     *
     * See KCompletionBase::setCompletionMode
     */
-    void setCompletionMode(KCompletion::CompletionMode mode) Q_DECL_OVERRIDE;
+    void setCompletionMode(KCompletion::CompletionMode mode) override;
 
     /**
      * Disables completion modes by makeing them non-checkable.
@@ -217,6 +207,7 @@ public:
      */
     void setCompletionModeDisabled(KCompletion::CompletionMode mode, bool disable = true);
 
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(4, 5)
     /**
      * Enables/disables the popup (context) menu.
      *
@@ -228,19 +219,20 @@ public:
      * Call this function with the argument set to false to disable the popup
      * menu.
      *
-     * @param showMenu If @p true, show the context menu.
+     * @param showMenu If @c true, show the context menu.
      * @deprecated since 4.5, use setContextMenuPolicy instead
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    virtual KCOMPLETION_DEPRECATED void setContextMenuEnabled(bool showMenu);
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use QWidget::setContextMenuPolicy(Qt::ContextMenuPolicy)")
+    virtual void setContextMenuEnabled(bool showMenu);
 #endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
-     * Returns @p true when the context menu is enabled.
+     * Returns @c true when the context menu is enabled.
      * @deprecated since 4.5, use contextMenuPolicy instead
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    KCOMPLETION_DEPRECATED bool isContextMenuEnabled() const;
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use QWidget::contextMenuPolicy()")
+    bool isContextMenuEnabled() const;
 #endif
 
     /**
@@ -249,21 +241,21 @@ public:
      * behavior of QLineEdit is used, which inserts the encoded URL.
      * Call setUrlDropsEnabled(false) if you need dropEvent to be called in a KLineEdit subclass.
      *
-     * @param enable If @p true, insert decoded URLs
+     * @param enable If @c true, insert decoded URLs
      */
     void setUrlDropsEnabled(bool enable);  // KF6: remove it and don't create LineEditUrlDropEventFilter by default.
 
     /**
-     * Returns @p true when decoded URL drops are enabled
+     * Returns @c true when decoded URL drops are enabled
      */
     bool urlDropsEnabled() const;
 
     /**
-     * By default, KLineEdit recognizes @p Key_Return and @p Key_Enter and emits
+     * By default, KLineEdit recognizes @c Key_Return and @c Key_Enter and emits
      * the returnPressed() signals, but it also lets the event pass,
      * for example causing a dialog's default-button to be called.
      *
-     * Call this method with @p trap = @p true to make @p KLineEdit stop these
+     * Call this method with @p trap = @c true to make @c KLineEdit stop these
      * events. The signals will still be emitted of course.
      *
      * @see trapReturnKey()
@@ -271,27 +263,27 @@ public:
     void setTrapReturnKey(bool trap);
 
     /**
-     * @returns @p true if keyevents of @p Key_Return or
-     * @p Key_Enter will be stopped or if they will be propagated.
+     * @returns @c true if keyevents of @c Key_Return or
+     * @c Key_Enter will be stopped or if they will be propagated.
      *
      * @see setTrapReturnKey ()
      */
     bool trapReturnKey() const;
 
     /**
-     * @returns the completion-box, that is used in completion mode
-     * CompletionPopup.
      * This method will create a completion-box if none is there, yet.
      *
      * @param create Set this to false if you don't want the box to be created
      *               i.e. to test if it is available.
+     * @returns the completion-box, that is used in completion mode
+     * CompletionPopup.
      */
     virtual KCompletionBox *completionBox(bool create = true);
 
     /**
      * Reimplemented for internal reasons, the API is not affected.
      */
-    void setCompletionObject(KCompletion *, bool handle = true) Q_DECL_OVERRIDE;
+    void setCompletionObject(KCompletion *, bool handle = true) override;
 
     /**
      * Reimplemented for internal reasons, the API is not affected.
@@ -342,40 +334,52 @@ public:
     */
     void setCompletionBox(KCompletionBox *box);
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * This makes the line edit display a grayed-out hinting text as long as
      * the user didn't enter any text. It is often used as indication about
      * the purpose of the line edit.
      * @deprecated since 5.0, use QLineEdit::setPlaceholderText instead.
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    KCOMPLETION_DEPRECATED void setClickMessage(const QString &msg);
+    KCOMPLETION_DEPRECATED_VERSION(5, 0, "Use QLineEdit::setPlaceholderText(const QString&)")
+    void setClickMessage(const QString &msg);
 #endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * @return the message set with setClickMessage
      * @deprecated since 5.0, use QLineEdit::placeholderText instead.
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
-    KCOMPLETION_DEPRECATED QString clickMessage() const;
+    KCOMPLETION_DEPRECATED_VERSION(5, 0, "Use QLineEdit::placeholderText()")
+    QString clickMessage() const;
 #endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 46)
     /**
      * This makes the line edit display an icon on one side of the line edit
      * which, when clicked, clears the contents of the line edit.
      * This is useful for such things as location or search bars.
-     **/
+     *
+     * @deprecated since 5.46 Use QLineEdit::setClearButtonEnabled
+     */
+    KCOMPLETION_DEPRECATED_VERSION(5, 46, "Use QLineEdit::setClearButtonEnabled(bool)")
     void setClearButtonShown(bool show);
+#endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 46)
     /**
      * @return whether or not the clear button is shown
-     **/
+     *
+     * @deprecated since 5.46 Use QLineEdit::isClearButtonEnabled
+     */
+    KCOMPLETION_DEPRECATED_VERSION(5, 46, "Use QLineEdit::isClearButtonEnabled()")
     bool isClearButtonShown() const;
+#endif
 
     /**
      * @return the size used by the clear button
      * @since 4.1
-     **/
+     */
     QSize clearButtonUsedSize() const;
 
     /**
@@ -398,7 +402,7 @@ Q_SIGNALS:
      * Emitted when the user presses the return key.
      *
      *  The argument is the current text. Note that this
-     * signal is @em not emitted if the widget's @p EchoMode is set to
+     * signal is @em not emitted if the widget's @c EchoMode is set to
      * QLineEdit::EchoMode.
      */
     void returnPressed(const QString &);
@@ -407,7 +411,7 @@ Q_SIGNALS:
      * Emitted when the completion key is pressed.
      *
      * Please note that this signal is @em not emitted if the
-     * completion mode is set to @p CompletionNone or @p EchoMode is
+     * completion mode is set to @c CompletionNone or @c EchoMode is
      * @em normal.
      */
     void completion(const QString &);
@@ -417,6 +421,7 @@ Q_SIGNALS:
      */
     void substringCompletion(const QString &);
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(4, 5)
     /**
      * Emitted when the text is changed NOT by the suggested autocompletion:
      * either when the user is physically typing keys, or when the text is changed programmatically,
@@ -429,7 +434,7 @@ Q_SIGNALS:
      * (by typing or accepting autocompletion), without side effects from
      * suggested autocompletion either. userTextChanged isn't needed anymore.
      */
-#ifndef KCOMPLETION_NO_DEPRECATED
+    KCOMPLETION_DEPRECATED_VERSION(4, 5, "Use QLineEdit::textEdited(const QString&)")
     QT_MOC_COMPAT void userTextChanged(const QString &);
 #endif
 
@@ -442,7 +447,7 @@ Q_SIGNALS:
      * KCompletionBase::setKeyBinding for details.
      *
      * Note that this signal is @em not emitted if the completion
-     * mode is set to @p CompletionNone or @p echoMode() is @em not  normal.
+     * mode is set to @c CompletionNone or @c echoMode() is @em not  normal.
      */
     void textRotation(KCompletionBase::KeyBindingType);
 
@@ -494,7 +499,7 @@ public Q_SLOTS:
     /**
      * See KCompletionBase::setCompletedText.
      */
-    void setCompletedText(const QString &) Q_DECL_OVERRIDE;
+    void setCompletedText(const QString &) override;
 
     /**
      * Same as the above function except it allows you to temporarily
@@ -504,7 +509,7 @@ public Q_SLOTS:
      * @param items list of completion matches to be shown in the completion box.
      * @param autoSuggest true if you want automatic text completion (suggestion) enabled.
      */
-    void setCompletedItems(const QStringList &items, bool autoSuggest = true) Q_DECL_OVERRIDE;
+    void setCompletedItems(const QStringList &items, bool autoSuggest = true) override;
 
     /**
      * Squeezes @p text into the line edit.
@@ -549,49 +554,49 @@ protected:
     /**
      * Reimplemented for internal reasons. API not affected.
      */
-    bool event(QEvent *) Q_DECL_OVERRIDE;
+    bool event(QEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QLineEdit::resizeEvent().
     */
-    void resizeEvent(QResizeEvent *) Q_DECL_OVERRIDE;
+    void resizeEvent(QResizeEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QLineEdit::keyPressEvent().
     */
-    void keyPressEvent(QKeyEvent *) Q_DECL_OVERRIDE;
+    void keyPressEvent(QKeyEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QLineEdit::mousePressEvent().
     */
-    void mousePressEvent(QMouseEvent *) Q_DECL_OVERRIDE;
+    void mousePressEvent(QMouseEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QLineEdit::mouseReleaseEvent().
     */
-    void mouseReleaseEvent(QMouseEvent *) Q_DECL_OVERRIDE;
+    void mouseReleaseEvent(QMouseEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QWidget::mouseDoubleClickEvent().
     */
-    void mouseDoubleClickEvent(QMouseEvent *) Q_DECL_OVERRIDE;
+    void mouseDoubleClickEvent(QMouseEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
     *
     * See QLineEdit::contextMenuEvent().
     */
-    void contextMenuEvent(QContextMenuEvent *) Q_DECL_OVERRIDE;
+    void contextMenuEvent(QContextMenuEvent *) override;
 
     /**
     * Reimplemented for internal reasons. API not affected.
@@ -621,17 +626,16 @@ protected:
     */
     bool autoSuggest() const;
 
-    void paintEvent(QPaintEvent *ev) Q_DECL_OVERRIDE;
+    void paintEvent(QPaintEvent *ev) override;
 
 private:
-    const QScopedPointer<KLineEditPrivate> d_ptr;
+    std::unique_ptr<KLineEditPrivate> const d_ptr;
 
     Q_PRIVATE_SLOT(d_func(), void _k_textChanged(const QString &))
     Q_PRIVATE_SLOT(d_func(), void _k_completionMenuActivated(QAction *))
     Q_PRIVATE_SLOT(d_func(), void _k_tripleClickTimeout())
     Q_PRIVATE_SLOT(d_func(), void _k_restoreSelectionColors())
     Q_PRIVATE_SLOT(d_func(), void _k_completionBoxTextChanged(const QString &))
-    Q_PRIVATE_SLOT(d_func(), void _k_updateClearButtonIcon(const QString &))
 };
 
 #endif
